@@ -90,7 +90,6 @@ _gdk_pixbuf_iterate (GdkPixbuf   *src,
 }
 
 
-
 /* -- desaturate -- */
 
 
@@ -128,7 +127,6 @@ _gdk_pixbuf_desaturate (GdkPixbuf *src,
 }
 
 
-
 /* -- invert -- */
 
 
@@ -153,7 +151,6 @@ _gdk_pixbuf_invert (GdkPixbuf *src,
 }
 
 
-
 /* -- brightness contrast -- */
 
 
@@ -263,7 +260,6 @@ _gdk_pixbuf_brightness_contrast (GdkPixbuf *src,
 }
 
 
-
 /* -- posterize -- */
 
 
@@ -324,7 +320,6 @@ _gdk_pixbuf_posterize (GdkPixbuf *src,
 }
 
 
-
 /* -- gimpcolorspace -- */
 
 
@@ -471,7 +466,6 @@ gimp_hls_to_rgb_int (gint *hue,
 }
 
 
-
 /* -- hue, lightness, saturation -- */
 
 
@@ -630,7 +624,6 @@ _gdk_pixbuf_hue_lightness_saturation (GdkPixbuf *src,
 }
 
 
-
 /*  -- color balance -- */
 
 
@@ -840,7 +833,6 @@ _gdk_pixbuf_color_balance (GdkPixbuf *src,
 }
 
 
-
 /* -- equalize histogram -- */
 
 typedef struct {
@@ -968,7 +960,6 @@ _gdk_pixbuf_eq_histogram (GdkPixbuf *src,
 }
 
 
-
 /* -- adjust levels -- */
 
 
@@ -1173,9 +1164,6 @@ _gdk_pixbuf_adjust_levels (GdkPixbuf *src,
 }
 
 
-
-
-
 typedef struct {
 	int      alpha;
 	guchar   lut[256][3];
@@ -1266,9 +1254,6 @@ _gdk_pixbuf_stretch_contrast (GdkPixbuf *src,
 }
 
 
-
-
-
 typedef struct {
 	int      alpha;
 	guchar   lut[256];
@@ -1440,7 +1425,7 @@ dither_step (GthPixbufOp *pixop)
 {
 	DitherData *data = pixop->data;
 	guchar      min, max, lightness;
-	int         red_pix, green_pix, blue_pix, alpha_pix;
+	int         red_pix, green_pix, blue_pix, alpha_pix = 0;
 	double     *error_pixel, *ngh_error_pixel;
 	int         dir = 1;
 
@@ -1592,15 +1577,19 @@ scale_step (GthPixbufOp *pixop)
 	} else if (data->keep_ratio) {
 		new_w = w;
 		new_h = h;
-		scale_keepping_ratio (&new_w, &new_h,
-				      data->width, data->height);
+		scale_keeping_ratio (&new_w, &new_h,
+				     data->width, data->height,
+				     TRUE);
 	} else {
 		new_w = data->width;
 		new_h = data->height;
 	}
 
-	if ((new_w > 0) && (new_h > 0))
-		pixop->dest = gdk_pixbuf_scale_simple (pixop->src, new_w, new_h, GDK_INTERP_BILINEAR);
+	if ((new_w > 1) && (new_h > 1))	{
+		if(pixop->dest != NULL)
+			g_object_unref(pixop->dest);
+		pixop->dest = _gdk_pixbuf_scale_simple_safe (pixop->src, new_w, new_h, GDK_INTERP_BILINEAR);
+	}
 }
 
 
