@@ -74,6 +74,13 @@ gfv_get_widget (GthFileView   *file_view)
 }
 
 
+static GtkWidget *
+gfv_get_drag_source (GthFileView   *file_view)
+{
+	return NULL;
+}
+
+
 /* To avoid excesive recomputes during insertion/deletion */
 
 
@@ -101,7 +108,7 @@ gfv_is_frozen (GthFileView  *file_view)
 
 static void
 gfv_insert (GthFileView  *file_view,
-	    int           pos, 
+	    int           pos,
 	    GdkPixbuf    *pixbuf,
 	    const char   *text,
 	    const char   *comment)
@@ -131,8 +138,8 @@ gfv_append_with_data (GthFileView  *file_view,
 
 
 static void
-gfv_remove (GthFileView  *file_view, 
-	    int           pos)
+gfv_remove (GthFileView  *file_view,
+	    gpointer      data)
 {
 }
 
@@ -151,13 +158,6 @@ gfv_set_image_pixbuf (GthFileView  *file_view,
 }
 
 
-static void
-gfv_set_unknown_pixbuf (GthFileView  *file_view,
-			int           pos)
-{
-}
-
-
 static void
 gfv_set_image_text (GthFileView  *file_view,
 		    int           pos,
@@ -255,20 +255,6 @@ gfv_pos_is_selected (GthFileView     *file_view,
 }
 
 
-static gboolean
-gfv_only_one_is_selected (GthFileView    *file_view)
-{
-	return FALSE;
-}
-
-
-static gboolean
-gfv_selection_not_null (GthFileView    *file_view)
-{
-	return FALSE;
-}
-
-
 static int
 gfv_get_first_selected (GthFileView *file_view)
 {
@@ -283,6 +269,13 @@ gfv_get_last_selected (GthFileView *file_view)
 }
 
 
+static int
+gfv_get_n_selected (GthFileView *file_view)
+{
+	return -1;
+}
+
+
 /* Setting spacing values */
 
 
@@ -298,21 +291,12 @@ gfv_set_image_width (GthFileView     *file_view,
 
 static void
 gfv_set_image_data (GthFileView     *file_view,
-		    int              pos, 
+		    int              pos,
 		    gpointer         data)
 {
 }
 
 
-static void
-gfv_set_image_data_full (GthFileView     *file_view,
-			 int              pos, 
-			 gpointer         data,
-			 GtkDestroyNotify destroy)
-{
-}
-
-
 static int
 gfv_find_image_from_data (GthFileView     *file_view,
 			  gpointer         data)
@@ -355,7 +339,7 @@ gfv_get_view_mode (GthFileView *file_view)
 
 static void
 gfv_moveto (GthFileView *file_view,
-	    int          pos, 
+	    int          pos,
 	    double       yalign)
 {
 }
@@ -370,8 +354,8 @@ gfv_image_is_visible (GthFileView *file_view,
 
 
 static int
-gfv_get_image_at (GthFileView *file_view, 
-		  int          x, 
+gfv_get_image_at (GthFileView *file_view,
+		  int          x,
 		  int          y)
 {
 	return -1;
@@ -392,6 +376,14 @@ gfv_get_last_visible (GthFileView *file_view)
 }
 
 
+static void
+gfv_set_visible_func (GthFileView    *file_view,
+                      GthVisibleFunc  func,
+                      gpointer        data)
+{
+}
+
+
 /* Sort */
 
 
@@ -413,14 +405,14 @@ gfv_unsorted (GthFileView *file_view)
 
 
 static void
-gfv_image_activated (GthFileView *file_view, 
+gfv_image_activated (GthFileView *file_view,
 		     int          pos)
 {
 }
 
 
 static void
-gfv_set_cursor (GthFileView *file_view, 
+gfv_set_cursor (GthFileView *file_view,
 		int          pos)
 {
 }
@@ -434,18 +426,42 @@ gfv_get_cursor (GthFileView *file_view)
 
 
 static void
-gfv_update_icon_theme (GthFileView *file_view)
+gfv_set_no_image_text (GthFileView *file_view,
+		       const char  *text)
 {
 }
 
 
 static void
-gfv_set_no_image_text (GthFileView *file_view,
-		       const char  *text)
+gfv_set_drag_dest_pos (GthFileView *file_view,
+		       int          x,
+		       int          y)
 {
 }
 
 
+static void
+gfv_get_drag_dest_pos (GthFileView     *file_view,
+		       int             *pos)
+{
+	*pos = -1;
+}
+
+
+static void
+gfv_set_reorderable (GthFileView  *file_view,
+		     gboolean      value)
+{
+}
+
+
+static gboolean
+gfv_get_reorderable (GthFileView  *file_view)
+{
+	return FALSE;
+}
+
+
 /* Interactive search */
 
 
@@ -481,6 +497,7 @@ gth_file_view_class_init (GthFileViewClass *file_view_class)
 	file_view_class->set_vadjustment      = gfv_set_vadjustment;
 	file_view_class->get_vadjustment      = gfv_get_vadjustment;
 	file_view_class->get_widget           = gfv_get_widget;
+	file_view_class->get_drag_source      = gfv_get_drag_source;
 	file_view_class->freeze               = gfv_freeze;
 	file_view_class->thaw                 = gfv_thaw;
 	file_view_class->is_frozen            = gfv_is_frozen;
@@ -490,7 +507,6 @@ gth_file_view_class_init (GthFileViewClass *file_view_class)
 	file_view_class->remove               = gfv_remove;
 	file_view_class->clear                = gfv_clear;
 	file_view_class->set_image_pixbuf     = gfv_set_image_pixbuf;
-	file_view_class->set_unknown_pixbuf   = gfv_set_unknown_pixbuf;
 	file_view_class->set_image_text       = gfv_set_image_text;
 	file_view_class->get_image_text       = gfv_get_image_text;
 	file_view_class->set_image_comment    = gfv_set_image_comment;
@@ -504,13 +520,11 @@ gth_file_view_class_init (GthFileViewClass *file_view_class)
 	file_view_class->unselect_all         = gfv_unselect_all;
 	file_view_class->get_file_list_selection = gfv_get_file_list_selection;
 	file_view_class->pos_is_selected      = gfv_pos_is_selected;
-	file_view_class->only_one_is_selected = gfv_only_one_is_selected;
-	file_view_class->selection_not_null   = gfv_selection_not_null;
 	file_view_class->get_first_selected   = gfv_get_first_selected;
 	file_view_class->get_last_selected    = gfv_get_last_selected;
+	file_view_class->get_n_selected       = gfv_get_n_selected;	
 	file_view_class->set_image_width      = gfv_set_image_width;
 	file_view_class->set_image_data       = gfv_set_image_data;
-	file_view_class->set_image_data_full  = gfv_set_image_data_full;
 	file_view_class->find_image_from_data = gfv_find_image_from_data;
 	file_view_class->get_image_data       = gfv_get_image_data;
 	file_view_class->enable_thumbs        = gfv_enable_thumbs;
@@ -521,13 +535,17 @@ gth_file_view_class_init (GthFileViewClass *file_view_class)
 	file_view_class->get_image_at         = gfv_get_image_at;
 	file_view_class->get_first_visible    = gfv_get_first_visible;
 	file_view_class->get_last_visible     = gfv_get_last_visible;
+	file_view_class->set_visible_func     = gfv_set_visible_func;
 	file_view_class->sorted               = gfv_sorted;
 	file_view_class->unsorted             = gfv_unsorted;
 	file_view_class->image_activated      = gfv_image_activated;
 	file_view_class->set_cursor           = gfv_set_cursor;
 	file_view_class->get_cursor           = gfv_get_cursor;
-	file_view_class->update_icon_theme    = gfv_update_icon_theme;
 	file_view_class->set_no_image_text    = gfv_set_no_image_text;
+	file_view_class->set_drag_dest_pos    = gfv_set_drag_dest_pos;
+	file_view_class->get_drag_dest_pos    = gfv_get_drag_dest_pos;
+	file_view_class->set_reorderable      = gfv_set_reorderable;
+	file_view_class->get_reorderable      = gfv_get_reorderable;
 	file_view_class->set_enable_search    = gfv_set_enable_search;
 	file_view_class->get_enable_search    = gfv_get_enable_search;
 
@@ -539,7 +557,7 @@ gth_file_view_class_init (GthFileViewClass *file_view_class)
 			      G_SIGNAL_RUN_FIRST,
 			      G_STRUCT_OFFSET (GthFileViewClass, selection_changed),
 			      NULL, NULL,
-			      gthumb_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 	file_view_signals[ITEM_ACTIVATED] =
 		g_signal_new ("item_activated",
@@ -636,6 +654,13 @@ gth_file_view_get_widget (GthFileView *file_view)
 }
 
 
+GtkWidget *
+gth_file_view_get_drag_source (GthFileView *file_view)
+{
+	return 	GTH_FILE_VIEW_GET_CLASS (file_view)->get_drag_source (file_view);
+}
+
+
 /* To avoid excesive recomputes during insertion/deletion */
 
 
@@ -665,7 +690,7 @@ gth_file_view_is_frozen (GthFileView  *file_view)
 
 void
 gth_file_view_insert (GthFileView  *file_view,
-		      int           pos, 
+		      int           pos,
 		      GdkPixbuf    *pixbuf,
 		      const char   *text,
 		      const char   *comment)
@@ -696,10 +721,10 @@ gth_file_view_append_with_data (GthFileView  *file_view,
 
 
 void
-gth_file_view_remove (GthFileView  *file_view, 
-		      int           pos)
+gth_file_view_remove (GthFileView  *file_view,
+		      gpointer      data)
 {
-	GTH_FILE_VIEW_GET_CLASS (file_view)->remove (file_view, pos);
+	GTH_FILE_VIEW_GET_CLASS (file_view)->remove (file_view, data);
 }
 
 
@@ -719,14 +744,6 @@ gth_file_view_set_image_pixbuf (GthFileView  *file_view,
 }
 
 
-void
-gth_file_view_set_unknown_pixbuf (GthFileView  *file_view,
-				  int           pos)
-{
-	GTH_FILE_VIEW_GET_CLASS (file_view)->set_unknown_pixbuf (file_view, pos);
-}
-
-
 void
 gth_file_view_set_image_text (GthFileView  *file_view,
 			      int           pos,
@@ -830,20 +847,6 @@ gth_file_view_pos_is_selected (GthFileView     *file_view,
 }
 
 
-gboolean
-gth_file_view_only_one_is_selected (GthFileView    *file_view)
-{
-	return GTH_FILE_VIEW_GET_CLASS (file_view)->only_one_is_selected (file_view);
-}
-
-
-gboolean
-gth_file_view_selection_not_null (GthFileView    *file_view)
-{
-	return GTH_FILE_VIEW_GET_CLASS (file_view)->selection_not_null (file_view);
-}
-
-
 int
 gth_file_view_get_first_selected (GthFileView *file_view)
 {
@@ -858,6 +861,13 @@ gth_file_view_get_last_selected (GthFileView *file_view)
 }
 
 
+int
+gth_file_view_get_n_selected (GthFileView *file_view)
+{
+	return GTH_FILE_VIEW_GET_CLASS (file_view)->get_n_selected (file_view);
+}
+
+
 /* Setting spacing values */
 
 
@@ -874,23 +884,13 @@ gth_file_view_set_image_width (GthFileView *file_view,
 
 void
 gth_file_view_set_image_data (GthFileView *file_view,
-			      int          pos, 
+			      int          pos,
 			      gpointer     data)
 {
 	GTH_FILE_VIEW_GET_CLASS (file_view)->set_image_data (file_view, pos, data);
 }
 
 
-void
-gth_file_view_set_image_data_full (GthFileView     *file_view,
-				   int              pos, 
-				   gpointer         data,
-				   GtkDestroyNotify destroy)
-{
-	GTH_FILE_VIEW_GET_CLASS (file_view)->set_image_data_full (file_view, pos, data, destroy);
-}
-
-
 int
 gth_file_view_find_image_from_data (GthFileView *file_view,
 				    gpointer     data)
@@ -935,7 +935,7 @@ gth_file_view_get_view_mode (GthFileView *file_view)
 
 void
 gth_file_view_moveto (GthFileView *file_view,
-		      int          pos, 
+		      int          pos,
 		      double       yalign)
 {
 	GTH_FILE_VIEW_GET_CLASS (file_view)->moveto (file_view, pos, yalign);
@@ -951,8 +951,8 @@ gth_file_view_image_is_visible (GthFileView *file_view,
 
 
 int
-gth_file_view_get_image_at (GthFileView *file_view, 
-			    int          x, 
+gth_file_view_get_image_at (GthFileView *file_view,
+			    int          x,
 			    int          y)
 {
 	return GTH_FILE_VIEW_GET_CLASS (file_view)->get_image_at (file_view, x, y);
@@ -973,6 +973,15 @@ gth_file_view_get_last_visible (GthFileView *file_view)
 }
 
 
+void
+gth_file_view_set_visible_func (GthFileView    *file_view,
+                                GthVisibleFunc  func,
+                                gpointer        data)
+{
+	GTH_FILE_VIEW_GET_CLASS (file_view)->set_visible_func (file_view, func, data);
+}
+
+
 /* Sort */
 
 
@@ -996,7 +1005,7 @@ gth_file_view_unsorted (GthFileView *file_view)
 
 
 void
-gth_file_view_image_activated (GthFileView *file_view, 
+gth_file_view_image_activated (GthFileView *file_view,
 			       int          pos)
 {
 	GTH_FILE_VIEW_GET_CLASS (file_view)->image_activated (file_view, pos);
@@ -1004,7 +1013,7 @@ gth_file_view_image_activated (GthFileView *file_view,
 
 
 void
-gth_file_view_set_cursor (GthFileView *file_view, 
+gth_file_view_set_cursor (GthFileView *file_view,
 			  int          pos)
 {
 	GTH_FILE_VIEW_GET_CLASS (file_view)->set_cursor (file_view, pos);
@@ -1018,13 +1027,6 @@ gth_file_view_get_cursor (GthFileView *file_view)
 }
 
 
-void
-gth_file_view_update_icon_theme (GthFileView *file_view)
-{
-	GTH_FILE_VIEW_GET_CLASS (file_view)->update_icon_theme (file_view);
-}
-
-
 void
 gth_file_view_set_no_image_text (GthFileView *file_view,
 				 const char  *text)
@@ -1033,6 +1035,41 @@ gth_file_view_set_no_image_text (GthFileView *file_view,
 }
 
 
+/* DnD */
+
+
+void
+gth_file_view_set_drag_dest_pos (GthFileView *file_view,
+				 int          x,
+				 int          y)
+{
+	GTH_FILE_VIEW_GET_CLASS (file_view)->set_drag_dest_pos (file_view, x, y);
+}
+
+
+void
+gth_file_view_get_drag_dest_pos (GthFileView     *file_view,
+				 int             *pos)
+{
+	GTH_FILE_VIEW_GET_CLASS (file_view)->get_drag_dest_pos (file_view, pos);
+}
+
+
+void
+gth_file_view_set_reorderable (GthFileView  *file_view,
+			       gboolean      value)
+{
+	GTH_FILE_VIEW_GET_CLASS (file_view)->set_reorderable (file_view, value);
+}
+
+
+gboolean
+gth_file_view_get_reorderable (GthFileView  *file_view)
+{
+	return GTH_FILE_VIEW_GET_CLASS (file_view)->get_reorderable (file_view);
+}
+
+
 /* Interactive search */
 
 
@@ -1059,7 +1096,7 @@ gth_file_view_selection_changed (GthFileView   *file_view)
 
 
 void
-gth_file_view_item_activated (GthFileView   *file_view, 
+gth_file_view_item_activated (GthFileView   *file_view,
 			      int            pos)
 {
 	g_signal_emit (file_view, file_view_signals[ITEM_ACTIVATED], 0, pos);
@@ -1067,7 +1104,7 @@ gth_file_view_item_activated (GthFileView   *file_view,
 
 
 void
-gth_file_view_cursor_changed (GthFileView   *file_view, 
+gth_file_view_cursor_changed (GthFileView   *file_view,
 			      int            pos)
 {
 	g_signal_emit (file_view, file_view_signals[CURSOR_CHANGED], 0, pos);
