@@ -121,10 +121,9 @@ typedef struct {
 	GList *        (* get_file_list_selection) (GthFileView  *file_view);
 	gboolean       (* pos_is_selected)      (GthFileView     *file_view,
 						 int              pos);
-	gboolean       (* only_one_is_selected) (GthFileView     *file_view);
-	gboolean       (* selection_not_null)   (GthFileView     *file_view);
 	int            (* get_first_selected)   (GthFileView     *file_view);
 	int            (* get_last_selected)    (GthFileView     *file_view);
+	int            (* get_n_selected)       (GthFileView     *file_view);
 
 	/* Setting spacing values */
 
@@ -136,10 +135,6 @@ typedef struct {
 	void           (* set_image_data)       (GthFileView     *file_view,
 						 int              pos,
 						 gpointer         data);
-	void           (* set_image_data_full)  (GthFileView     *file_view,
-						 int              pos,
-						 gpointer         data,
-						 GtkDestroyNotify destroy);
 	int            (* find_image_from_data) (GthFileView     *file_view,
 						 gpointer         data);
 	gpointer       (* get_image_data)       (GthFileView     *file_view,
@@ -180,7 +175,6 @@ typedef struct {
 	void           (* set_cursor)           (GthFileView *file_view,
 						 int          pos);
 	int            (* get_cursor)           (GthFileView *file_view);
-	void           (* update_icon_theme)    (GthFileView *file_view);
 	void           (* set_no_image_text)    (GthFileView *file_view,
 						 const char  *text);
 
@@ -269,10 +263,9 @@ void           gth_file_view_unselect_all         (GthFileView     *file_view);
 GList *        gth_file_view_get_file_list_selection (GthFileView *file_view);
 gboolean       gth_file_view_pos_is_selected      (GthFileView     *file_view,
 						   int              pos);
-gboolean       gth_file_view_only_one_is_selected (GthFileView    *file_view);
-gboolean       gth_file_view_selection_not_null   (GthFileView    *file_view);
 int            gth_file_view_get_first_selected   (GthFileView    *file_view);
 int            gth_file_view_get_last_selected    (GthFileView    *file_view);
+int            gth_file_view_get_n_selected       (GthFileView    *file_view);
 
 /* Setting spacing values */
 
@@ -284,10 +277,6 @@ void           gth_file_view_set_image_width     (GthFileView     *file_view,
 void           gth_file_view_set_image_data       (GthFileView     *file_view,
 						   int              pos,
 						   gpointer         data);
-void           gth_file_view_set_image_data_full  (GthFileView     *file_view,
-						   int              pos,
-						   gpointer         data,
-						   GtkDestroyNotify destroy);
 int            gth_file_view_find_image_from_data (GthFileView     *file_view,
 						   gpointer         data);
 gpointer       gth_file_view_get_image_data       (GthFileView     *file_view,
@@ -331,7 +320,6 @@ void           gth_file_view_image_activated      (GthFileView *file_view,
 void           gth_file_view_set_cursor           (GthFileView *file_view,
 						   int          pos);
 int            gth_file_view_get_cursor           (GthFileView *file_view);
-void           gth_file_view_update_icon_theme    (GthFileView *file_view);
 void           gth_file_view_set_no_image_text    (GthFileView *file_view,
 						   const char  *text);
 
