@@ -36,6 +36,7 @@
 #define GTHUMB_STOCK_ENHANCE              "gthumb-enhance"
 #define GTHUMB_STOCK_EDIT_IMAGE           "gthumb-edit-image"
 #define GTHUMB_STOCK_FILM                 "gthumb-film"
+#define GTHUMB_STOCK_FILTER               "gthumb-filter"
 #define GTHUMB_STOCK_FLIP                 "gthumb-flip"
 #define GTHUMB_STOCK_HISTOGRAM            "gthumb-histogram"
 #define GTHUMB_STOCK_HUE_SATURATION       "gthumb-hue-saturation"
@@ -44,6 +45,7 @@
 #define GTHUMB_STOCK_INVERT               "gthumb-invert"
 #define GTHUMB_STOCK_LEVELS               "gthumb-levels"
 #define GTHUMB_STOCK_LIBRARY              "gthumb-library"
+#define GTHUMB_STOCK_METADATA             "gthumb-metadata"
 #define GTHUMB_STOCK_MIRROR               "gthumb-mirror"
 #define GTHUMB_STOCK_NEXT_IMAGE           "gthumb-next-image"
 #define GTHUMB_STOCK_POSTERIZE            "gthumb-posterize"
@@ -62,7 +64,7 @@
 #define GTHUMB_STOCK_SWAP                 "gthumb-swap"
 #define GTHUMB_STOCK_THRESHOLD            "gthumb-threshold"
 #define GTHUMB_STOCK_TRANSFORM            "gthumb-transform"
-#define GTHUMB_STOCK_ZOOM_WIDTH		  "gthumb-zoom-width"
+#define GTHUMB_STOCK_ZOOM_WIDTH		  "zoom-fit-width"
 
 
 void gthumb_stock_init (void);
