@@ -640,8 +640,7 @@ comp_func_exif_date (gconstpointer a,
 	data_a = IMAGE_DATA (a);
 	data_b = IMAGE_DATA (b);
 
-	return gth_sort_by_filetime_then_name (data_a->exif_time, data_b->exif_time,
-					       data_a->src_file->path, data_b->src_file->path);
+	return gth_sort_by_exiftime_then_name (data_a->src_file, data_b->src_file);
 }
 
 
@@ -695,6 +694,14 @@ get_sortfunc (CatalogWebExporter *ce)
 }
 
 
+#define RETURN_IMAGE_FIELD(image, field) {	\
+	if (image == NULL)			\
+		return 0;			\
+	else					\
+		return image->field;		\
+}
+
+
 static int
 get_var_value (const char *var_name,
 	       gpointer    data)
@@ -717,17 +724,17 @@ get_var_value (const char *var_name,
 		return GTH_VISIBILITY_ALWAYS;
 
 	else if (strcmp (var_name, "image_width") == 0)
-		return ce->eval_image->image_width;
+		RETURN_IMAGE_FIELD (ce->eval_image, image_width)
 	else if (strcmp (var_name, "image_height") == 0)
-		return ce->eval_image->image_height;
+		RETURN_IMAGE_FIELD (ce->eval_image, image_height)
 	else if (strcmp (var_name, "preview_width") == 0)
-		return ce->eval_image->preview_width;
+		RETURN_IMAGE_FIELD (ce->eval_image, preview_width)
 	else if (strcmp (var_name, "preview_height") == 0)
-		return ce->eval_image->preview_height;
+		RETURN_IMAGE_FIELD (ce->eval_image, preview_height)
 	else if (strcmp (var_name, "thumb_width") == 0)
-		return ce->eval_image->thumb_width;
+		RETURN_IMAGE_FIELD (ce->eval_image, thumb_width)
 	else if (strcmp (var_name, "thumb_height") == 0)
-		return ce->eval_image->thumb_height;
+		RETURN_IMAGE_FIELD (ce->eval_image, thumb_height)
 
 	else if (strcmp (var_name, "image_dim_visibility_index") == 0)
 		return ce->index_caption_mask & GTH_CAPTION_IMAGE_DIM;
@@ -1201,10 +1208,11 @@ gth_parsed_doc_print (GList              *document,
 
 			max_size = gth_tag_get_var (ce, tag, "max_size");
 			if (max_size > 0)
-				scale_keepping_ratio (&image_width,
-						      &image_height,
-						      max_size,
-						      max_size);
+				scale_keeping_ratio (&image_width,
+						     &image_height,
+						     max_size,
+						     max_size,
+						     FALSE);
 
 			image_src_relative = get_path_relative_to_uri (image_src, ce->location);
 			
@@ -1303,6 +1311,7 @@ gth_parsed_doc_print (GList              *document,
 			idx = get_image_idx (tag, ce);
 			idata = g_list_nth (ce->file_list, idx)->data;
 			ce->eval_image = idata;
+			
 			uri = get_image_uri (ce, idata, ce->location);
 			if (gth_tag_get_var (ce, tag, "relative_path") != 0) {
 				char *tmp;
@@ -1383,6 +1392,7 @@ gth_parsed_doc_print (GList              *document,
 			idx = get_image_idx (tag, ce);
 			idata = g_list_nth (ce->file_list, idx)->data;
 			ce->eval_image = idata;
+			
 			if (idata->date_time == NULL)
 				break;
 
@@ -1460,7 +1470,6 @@ gth_parsed_doc_print (GList              *document,
 					}
 				}
 				write_line ("  </tr>\n", fout);
-
 			}
 			break;
 
@@ -1582,6 +1591,10 @@ gth_parsed_doc_print (GList              *document,
 			break;
 
 		case GTH_TAG_IF:
+			idx = MIN (ce->image, ce->n_images - 1);
+			idata = g_list_nth (ce->file_list, idx)->data;
+			ce->eval_image = idata;
+			
 			for (scan = tag->value.cond_list; scan; scan = scan->next) {
 				GthCondition *cond = scan->data;
 				if (expression_value (ce, cond->expr) != 0) {
@@ -2011,6 +2024,7 @@ export__save_html_files (CatalogWebExporter *ce)
 	}
 
 	ce->image = 0;
+	ce->eval_image = NULL;
 	ce->page = 0;
 	ce->saving_timeout = g_timeout_add (SAVING_TIMEOUT,
 					    save_html_index_cb,
@@ -2285,7 +2299,7 @@ image_loader_done (ImageLoader *iloader,
 	if (ce->copy_images && ce->resize_images) {
 		int w = gdk_pixbuf_get_width (pixbuf);
 		int h = gdk_pixbuf_get_height (pixbuf);
-		if (scale_keepping_ratio (&w, &h, ce->resize_max_width, ce->resize_max_height)) {
+		if (scale_keeping_ratio (&w, &h, ce->resize_max_width, ce->resize_max_height, FALSE)) {
 			GdkPixbuf *scaled;
 			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->image);
@@ -2305,9 +2319,11 @@ image_loader_done (ImageLoader *iloader,
 		int w = gdk_pixbuf_get_width (pixbuf);
 		int h = gdk_pixbuf_get_height (pixbuf);
 
-		if (scale_keepping_ratio (&w, &h,
-					  ce->preview_max_width,
-					  ce->preview_max_height)) {
+		if (scale_keeping_ratio (&w, &h,
+					 ce->preview_max_width,
+					 ce->preview_max_height,
+					 FALSE)) 
+		{
 			GdkPixbuf *scaled;
 			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->preview);
@@ -2336,9 +2352,11 @@ image_loader_done (ImageLoader *iloader,
 		int w = gdk_pixbuf_get_width (pixbuf);
 		int h = gdk_pixbuf_get_height (pixbuf);
 
-		if (scale_keepping_ratio (&w, &h,
-					  ce->thumb_width,
-					  ce->thumb_height)) {
+		if (scale_keeping_ratio (&w, &h,
+					 ce->thumb_width,
+					 ce->thumb_height,
+					 FALSE)) 
+		{
 			GdkPixbuf *scaled;
 			scaled = pixbuf_scale (pixbuf, w, h, GDK_INTERP_BILINEAR);
 			g_object_unref (idata->thumb);
