@@ -33,6 +33,7 @@
 #include "file-data.h"
 #include "gth-exif-utils.h"
 #include "file-utils.h"
+#include "gth-utils.h"
 #include "gth-window.h"
 #include "main.h"
 
@@ -75,8 +76,6 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
-#ifdef HAVE_LIBEXIF
-
 static gboolean
 exif_time_available (DialogData *data)
 {
@@ -90,12 +89,10 @@ exif_time_available (DialogData *data)
 
 	fd = data->file_list->data;
 
-	return get_exif_time (fd->path) != 0;
+	return get_metadata_time (fd->mime_type, fd->path) != 0;
 }
 
 
-#endif /* HAVE_LIBEXIF */
-
 #define is_active(x) (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (x)))
 
 
@@ -123,11 +120,9 @@ ok_clicked (GtkWidget  *button,
 		if (is_active (data->cd_created_radiobutton)) {
 			mtime = get_file_ctime (fdata->path);
 			comment_time = mtime;
-#ifdef HAVE_LIBEXIF
 		} else if (is_active (data->cd_exif_radiobutton)) {
-			mtime = get_exif_time (fdata->path);
+			mtime = get_metadata_time (fdata->mime_type, fdata->path);
 			comment_time = mtime;
-#endif /* HAVE_LIBEXIF */
 		} else if (is_active (data->cd_adjust_timezone_radiobutton)) {
 			time_t tz;
 
@@ -199,31 +194,7 @@ static void
 help_cb (GtkWidget  *widget, 
 	 DialogData *data)
 {
-	GError *err;
-
-	err = NULL;  
-	gnome_help_display ("gthumb", "gthumb-change-date", &err);
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-change-date");
 }
 
 
@@ -280,11 +251,7 @@ dlg_change_date (GthWindow *window)
 
 	/* Set widgets data. */
 
-#ifndef HAVE_LIBEXIF
-	gtk_widget_hide (data->cd_exif_radiobutton);
-#else /* HAVE_LIBEXIF */
 	gtk_widget_set_sensitive (data->cd_exif_radiobutton, exif_time_available (data));
-#endif /* HAVE_LIBEXIF */
 
 	gtk_widget_set_sensitive (data->cd_dateedit, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->cd_following_date_radiobutton)));
 
