@@ -22,12 +22,12 @@
 
 #include <config.h>
 #include <string.h>
+#include <time.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-dateedit.h>
 #include <glade/glade.h>
 
@@ -35,13 +35,15 @@
 #include "preferences.h"
 #include "main.h"
 #include "glib-utils.h"
-#include "gthumb-window.h"
+#include "gth-utils.h"
+#include "gth-window.h"
 #include "gtk-utils.h"
 #include "gth-file-view.h"
 #include "comments.h"
 #include "gth-exif-utils.h"
 #include "dlg-comment.h"
 #include "dlg-image-prop.h"
+#include "file-utils.h"
 
 
 enum {
@@ -61,7 +63,7 @@ enum {
 
 
 typedef struct {
-	GThumbWindow  *window;
+	GthWindow     *window;
 	GladeXML      *gui;
 	GList         *file_list;
 
@@ -73,7 +75,7 @@ typedef struct {
 	GtkWidget     *date_optionmenu;
 	GtkWidget     *date_dateedit;
 	GtkWidget     *save_changed_checkbutton;
-	GtkWidget     *save_button;
+	GtkWidget     *ok_button;
 
 	CommentData   *original_cdata;
 	gboolean       have_exif_data;
@@ -102,7 +104,7 @@ static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	data->window->comment_dlg = NULL;
+	gth_window_set_comment_dlg (GTH_WINDOW (data->window), NULL);
 	g_object_unref (data->gui);
 	free_dialog_data (data);
 	g_free (data);
@@ -118,20 +120,6 @@ unrealize_cb (GtkWidget  *widget,
 }
 
 
-static int
-strcmp_null_tollerant (const char *s1, const char *s2)
-{
-	if ((s1 == NULL) && (s2 == NULL))
-		return 0;
-	else if ((s1 != NULL) && (s2 == NULL))
-		return 1;
-	else if ((s1 == NULL) && (s2 != NULL))
-		return -1;
-	else 
-		return strcmp (s1, s2);
-}
-
-
 static int
 text_field_cmp (const char *s1, const char *s2)
 {
@@ -140,7 +128,7 @@ text_field_cmp (const char *s1, const char *s2)
 	if ((s2 != NULL) && (*s2 == 0))
 		s2 = NULL;
 
-	return strcmp_null_tollerant (s1, s2);
+	return strcmp_null_tolerant (s1, s2);
 }
 
 
@@ -161,9 +149,7 @@ get_requested_time (DialogData *data,
 		t = time (NULL);
 		break;
 	case EXIF_DATE:
-#ifdef HAVE_LIBEXIF
-		t = get_exif_time (filename);
-#endif
+		t = get_metadata_time (NULL, filename);
 		break;
 	case LAST_MODIFIED_DATE:
 		t = get_file_mtime (filename);
@@ -208,7 +194,7 @@ save_clicked_cb (GtkWidget  *widget,
 						 &start_iter, 
 						 &end_iter, 
 						 FALSE);
-	if (comment_text)
+	if (comment_text != NULL)
 		cdata->comment = comment_text;
 
 	/**/
@@ -273,13 +259,15 @@ save_clicked_cb (GtkWidget  *widget,
 			comments_save_comment_non_null (filename, cdata);
 		else
 			comments_save_comment (filename, cdata);
-		all_windows_notify_update_comment (filename); 
+		all_windows_notify_update_metadata (filename); 
 	}
 
-	if ((data->window != NULL) && (data->window->image_prop_dlg != NULL))
-		dlg_image_prop_update (data->window->image_prop_dlg);
-
+	gth_window_update_current_image_metadata (data->window);
 	comment_data_free (cdata);
+
+	/**/
+
+	dlg_comment_close (data->dialog);
 }
 
 
@@ -288,31 +276,7 @@ static void
 help_cb (GtkWidget  *widget, 
 	 DialogData *data)
 {
-	GError *err;
-
-	err = NULL;  
-	gnome_help_display ("gthumb", "gthumb-comments", &err);
-	
-	if (err != NULL) {
-		GtkWidget *dialog;
-		
-		dialog = gtk_message_dialog_new (GTK_WINDOW (data->dialog),
-						 0,
-						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
-						 _("Could not display help: %s"),
-						 err->message);
-		
-		g_signal_connect (G_OBJECT (dialog), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  NULL);
-		
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-		
-		gtk_widget_show (dialog);
-		
-		g_error_free (err);
-	}
+	gthumb_display_help (GTK_WINDOW (data->dialog), "gthumb-comments");
 }
 
 
@@ -344,22 +308,10 @@ date_optionmenu_changed_cb (GtkOptionMenu *option_menu,
 					  get_file_mtime (first_image));
 		break;
 	case EXIF_DATE:
-#ifdef HAVE_LIBEXIF
 		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit),
-					  get_exif_time (first_image));
-#endif
+					  get_metadata_time (NULL, first_image));
 		break;
 	}
-
-	gtk_widget_set_sensitive (data->save_button, TRUE);
-}
-
-
-static void
-editabled_changed_cb (GtkEditable *editable,
-		      DialogData  *data)
-{
-	gtk_widget_set_sensitive (data->save_button, TRUE);
 }
 
 
@@ -383,17 +335,18 @@ get_exif_date_option_item (DialogData *data)
 }
 
 
-
 GtkWidget *
-dlg_comment_new (GThumbWindow *window)
+dlg_comment_new (GthWindow *window)
 {
-	DialogData   *data;
-	GtkWidget    *btn_close;
-	GtkWidget    *btn_help;
-
-	if (window->comment_dlg != NULL) {
-		gtk_window_present (GTK_WINDOW (window->comment_dlg));
-		return window->comment_dlg;
+	GtkWidget  *current_dialog;
+	DialogData *data;
+	GtkWidget  *btn_close;
+	GtkWidget  *btn_help;
+	
+	current_dialog = gth_window_get_comment_dlg (window);
+	if (current_dialog != NULL) {
+		gtk_window_present (GTK_WINDOW (current_dialog));
+		return current_dialog;
 	}
 
 	data = g_new0 (DialogData, 1);
@@ -408,15 +361,16 @@ dlg_comment_new (GThumbWindow *window)
 
 	/* Get the widgets. */
 
-	window->comment_dlg = data->dialog = glade_xml_get_widget (data->gui, "comments_dialog");
+	data->dialog = glade_xml_get_widget (data->gui, "comments_dialog");
+	gth_window_set_comment_dlg (window, data->dialog);
 	data->comment_main_box = glade_xml_get_widget (data->gui, "comment_main_box");
 	data->note_text_view = glade_xml_get_widget (data->gui, "note_text");
 	data->place_entry = glade_xml_get_widget (data->gui, "place_entry");
 	data->date_optionmenu = glade_xml_get_widget (data->gui, "date_optionmenu");
 	data->date_dateedit = glade_xml_get_widget (data->gui, "date_dateedit");
 	data->save_changed_checkbutton = glade_xml_get_widget (data->gui, "save_changed_checkbutton");
-	data->save_button = glade_xml_get_widget (data->gui, "save_button");
-	btn_close = glade_xml_get_widget (data->gui, "close_button");
+	data->ok_button = glade_xml_get_widget (data->gui, "ok_button");
+	btn_close = glade_xml_get_widget (data->gui, "cancel_button");
 	btn_help = glade_xml_get_widget (data->gui, "help_button");
 
 	data->note_text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (data->note_text_view));
@@ -433,7 +387,7 @@ dlg_comment_new (GThumbWindow *window)
 			  "unrealize",
 			  G_CALLBACK (unrealize_cb),
 			  data);
-	g_signal_connect (G_OBJECT (data->save_button), 
+	g_signal_connect (G_OBJECT (data->ok_button), 
 			  "clicked",
 			  G_CALLBACK (save_clicked_cb),
 			  data);
@@ -451,26 +405,11 @@ dlg_comment_new (GThumbWindow *window)
 			  G_CALLBACK (date_optionmenu_changed_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (data->place_entry), 
-			  "changed",
-			  G_CALLBACK (editabled_changed_cb),
-			  data);
-	g_signal_connect (G_OBJECT (data->note_text_buffer), 
-			  "changed",
-			  G_CALLBACK (editabled_changed_cb),
-			  data);
-
 	/* run dialog. */
 
 	gtk_widget_grab_focus (data->note_text_view);
-
-	{
-		GtkWidget *parent_win = window->viewer;
-		while (parent_win != NULL && !GTK_IS_WINDOW(parent_win))
-			parent_win = parent_win->parent;
-		if (parent_win != NULL)
-			gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (parent_win));
-	}
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
+				      GTK_WINDOW (window));
 
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	pref_util_restore_window_geometry (GTK_WINDOW (data->dialog), DIALOG_NAME);
@@ -497,10 +436,10 @@ dlg_comment_update (GtkWidget *dlg)
 	
 	free_dialog_data (data);
 
-	data->file_list = gth_file_view_get_file_list_selection (data->window->file_list->view);
+	data->file_list = gth_window_get_file_list_selection (data->window); 
 
 	if (data->file_list == NULL) {
-		gtk_widget_set_sensitive (data->save_button, FALSE);
+		gtk_widget_set_sensitive (data->ok_button, FALSE);
 		gtk_widget_set_sensitive (data->comment_main_box, FALSE);
 		return;
 	} else 
@@ -522,19 +461,16 @@ dlg_comment_update (GtkWidget *dlg)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->save_changed_checkbutton), FALSE);
 	}
 
-	/**/
-
-#ifndef HAVE_LIBEXIF
+	/* Does at least one image have exif data? */
 	data->have_exif_data = FALSE;
-#else
-	{
-		char *first_image = data->file_list->data;
-		data->have_exif_data = have_exif_data (first_image);
-	}
-#endif
+	for (scan = data->file_list; scan; scan = scan->next)
+		if (have_exif_time (scan->data)) {
+			data->have_exif_data = TRUE;
+			break;
+		}
 	
 	first_image = data->file_list->data;
-	data->original_cdata = cdata = comments_load_comment (first_image);
+	data->original_cdata = cdata = comments_load_comment (first_image, TRUE);
 
 	if (cdata != NULL) {
 		comment_data_free_keywords (cdata);
@@ -543,13 +479,13 @@ dlg_comment_update (GtkWidget *dlg)
 		for (scan = data->file_list->next; scan; scan = scan->next) {
 			CommentData *scan_cdata;
 
-			scan_cdata = comments_load_comment (scan->data);
+			scan_cdata = comments_load_comment (scan->data, TRUE);
 
 			/* If there is no comment then all fields must be
 			 * considered to differ. */
 
 			if ((scan_cdata == NULL) 
-			    || strcmp_null_tollerant (cdata->place, 
+			    || strcmp_null_tolerant (cdata->place, 
 						      scan_cdata->place) != 0) {
 				g_free (cdata->place);
 				cdata->place = NULL;
@@ -560,7 +496,7 @@ dlg_comment_update (GtkWidget *dlg)
 				cdata->time = 0;
 			
 			if ((scan_cdata == NULL) 
-			    || strcmp_null_tollerant (cdata->comment, 
+			    || strcmp_null_tolerant (cdata->comment, 
 						      scan_cdata->comment) != 0) {
 				g_free (cdata->comment);
 				cdata->comment = NULL;
@@ -614,8 +550,6 @@ dlg_comment_update (GtkWidget *dlg)
 		gtk_option_menu_set_history (GTK_OPTION_MENU (data->date_optionmenu), NO_DATE);
 		gnome_date_edit_set_time (GNOME_DATE_EDIT (data->date_dateedit), ctime);
 	}
-
-	gtk_widget_set_sensitive (data->save_button, FALSE);
 }
 
 
