@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,7 +31,9 @@
 
 #include "async-pixbuf-ops.h"
 #include "gth-pixbuf-op.h"
-#include "gthumb-window.h"
+#include "gth-window.h"
+#include "gthumb-stock.h"
+#include "glib-utils.h"
 #include "pixbuf-utils.h"
 
 
@@ -41,13 +43,15 @@
 
 
 typedef struct {
-	GThumbWindow *window;
+	GthWindow    *window;
 	ImageViewer  *viewer;
 	GdkPixbuf    *image;
 	GdkPixbuf    *orig_pixbuf;
 	GdkPixbuf    *new_pixbuf;
 
+	GthPixbufOp  *pixop;
 	gboolean      modified;
+	gboolean      dialog_preview;
 
 	GladeXML     *gui;
 
@@ -61,9 +65,14 @@ typedef struct {
 
 /* called when the main dialog is closed. */
 static void
-destroy_cb (GtkWidget  *widget, 
+destroy_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
+	if (data->pixop != NULL) {
+		if (data->dialog_preview)
+			g_signal_handlers_disconnect_by_data (data->pixop, data);
+		g_object_unref (data->pixop);
+	}
 	g_object_unref (data->image);
 	g_object_unref (data->orig_pixbuf);
 	g_object_unref (data->new_pixbuf);
@@ -77,82 +86,84 @@ preview_done_cb (GthPixbufOp *pixop,
 		 gboolean     completed,
 		 DialogData  *data)
 {
-	gtk_widget_queue_draw (data->bc_preview_image);
-	g_object_unref (pixop);
+	if (completed)
+		gtk_widget_queue_draw (data->bc_preview_image);
+	g_object_unref (data->pixop);
+	data->pixop = NULL;
+
 	return FALSE;
 }
 
 
 static void
-apply_changes (DialogData *data, 
+apply_changes (DialogData *data,
 	       GdkPixbuf  *src, 
 	       GdkPixbuf  *dest, 
-	       gboolean    preview)
+	       gboolean    dialog_preview,
+	       gboolean    window_preview)
 {
-	double       hue;
-	double       lightness;
-	double       saturation;
-	GthPixbufOp *pixop;
+	double hue;
+	double lightness;
+	double saturation;
+
+	if (data->pixop != NULL) {
+		gth_pixbuf_op_stop (data->pixop);
+		g_object_unref (data->pixop);
+	}
 
 	hue = gtk_range_get_value (GTK_RANGE (data->bc_hue_hscale));
 	lightness = gtk_range_get_value (GTK_RANGE (data->bc_lightness_hscale));
 	saturation = gtk_range_get_value (GTK_RANGE (data->bc_saturation_hscale));
 
-	pixop = _gdk_pixbuf_hue_lightness_saturation (src,
-						      dest,
-						      hue,
-						      lightness,
-						      saturation);
+	data->pixop = _gdk_pixbuf_hue_lightness_saturation (src,
+							    dest,
+							    hue,
+							    lightness,
+							    saturation);
 
-	if (preview) {
-		g_signal_connect (G_OBJECT (pixop),
-				  "done",
+	data->dialog_preview = dialog_preview;
+	if (dialog_preview) {
+		g_signal_connect (G_OBJECT (data->pixop),
+				  "pixbuf_op_done",
 				  G_CALLBACK (preview_done_cb),
 				  data);
-		gth_pixbuf_op_start (pixop);
-	} else {
-		window_exec_pixbuf_op (data->window, pixop);
-		g_object_unref (pixop);
-	}		
+		gth_pixbuf_op_start (data->pixop);
+	}
+	else
+		gth_window_exec_pixbuf_op (data->window, data->pixop, window_preview);
 }
 
 
 /* called when the ok button is clicked. */
 static void
-ok_cb (GtkWidget  *widget, 
+ok_cb (GtkWidget  *widget,
        DialogData *data)
 {
-	apply_changes (data, data->image, data->image, FALSE);
-	/*
-	image_viewer_set_pixbuf (data->viewer, data->image);
-	window_image_modified (data->window, TRUE);
-	*/
+	apply_changes (data, data->image, data->image, FALSE, FALSE);
 	gtk_widget_destroy (data->dialog);
 }
 
 
 /* called when the cancel button is clicked. */
 static void
-cancel_cb (GtkWidget  *widget, 
+cancel_cb (GtkWidget  *widget,
 	   DialogData *data)
 {
-	if (data->modified) {
+	if (data->modified)
 		image_viewer_set_pixbuf (data->viewer, data->image);
-		window_image_modified (data->window, FALSE);
-	}
 	gtk_widget_destroy (data->dialog);
 }
 
 
 /* called when the preview button is clicked. */
 static void
-preview_cb (GtkWidget  *widget, 
+preview_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
 	GdkPixbuf *preview;
 
 	preview = gdk_pixbuf_copy (data->image);
-	apply_changes (data, preview, preview, FALSE);
+	apply_changes (data, preview, preview, FALSE, TRUE);
 	g_object_unref (preview);
 
 	data->modified = TRUE;
@@ -161,7 +172,7 @@ preview_cb (GtkWidget  *widget,
 
 /* called when the revert button is clicked. */
 static void
-reset_cb (GtkWidget  *widget, 
+reset_cb (GtkWidget  *widget,
 	  DialogData *data)
 {
 	gtk_range_set_value (GTK_RANGE (data->bc_hue_hscale), 0.0);
@@ -174,17 +185,17 @@ static void
 range_value_changed (GtkRange   *range,
 		     DialogData *data)
 {
-	apply_changes (data, data->orig_pixbuf, data->new_pixbuf, TRUE);
+	apply_changes (data, data->orig_pixbuf, data->new_pixbuf, TRUE, FALSE);
 }
 
 
 static GtkWidget*
 gimp_scale_entry_new (GtkWidget  *parent_box,
 		      gfloat      value,
-                      gfloat      lower,
-                      gfloat      upper,
-                      gfloat      step_increment,
-                      gfloat      page_increment)
+		      gfloat      lower,
+		      gfloat      upper,
+		      gfloat      step_increment,
+		      gfloat      page_increment)
 {
 	GtkWidget *hbox;
 	GtkWidget *scale;
@@ -194,7 +205,7 @@ gimp_scale_entry_new (GtkWidget  *parent_box,
 	adj = gtk_adjustment_new (value, lower, upper,
 				  step_increment, page_increment,
 				  0.0);
-	
+
 	spinbutton = gtk_spin_button_new  (GTK_ADJUSTMENT (adj), 1.0, 0);
 	scale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
 	gtk_scale_set_draw_value (GTK_SCALE (scale), FALSE);
@@ -213,7 +224,7 @@ gimp_scale_entry_new (GtkWidget  *parent_box,
 
 
 void
-dlg_hue_saturation (GThumbWindow *window)
+dlg_hue_saturation (GthWindow *window)
 {
 	DialogData *data;
 	GtkWidget  *ok_button;
@@ -221,6 +232,7 @@ dlg_hue_saturation (GThumbWindow *window)
 	GtkWidget  *cancel_button;
 	GtkWidget  *preview_button;
 	GtkWidget  *hbox;
+	GtkWidget  *reset_image;
 	GdkPixbuf  *image;
 	int         image_width, image_height;
 	int         preview_width, preview_height;
@@ -247,27 +259,30 @@ dlg_hue_saturation (GThumbWindow *window)
 	preview_button = glade_xml_get_widget (data->gui, "hl_preview_button");
 
 	hbox = glade_xml_get_widget (data->gui, "hl_hue_hbox");
-	data->bc_hue_hscale = gimp_scale_entry_new (hbox, 
+	data->bc_hue_hscale = gimp_scale_entry_new (hbox,
 						    0.0, 
 						    -180.0, 180.0,
 						    1.0,
 						    15.0);
 
 	hbox = glade_xml_get_widget (data->gui, "hl_lightness_hbox");
-	data->bc_lightness_hscale = gimp_scale_entry_new (hbox, 
+	data->bc_lightness_hscale = gimp_scale_entry_new (hbox,
 							  0.0, 
 							  -100.0, 100.0,
 							  1.0,
 							  10.0);
 
 	hbox = glade_xml_get_widget (data->gui, "hl_saturation_hbox");
-	data->bc_saturation_hscale = gimp_scale_entry_new (hbox, 
+	data->bc_saturation_hscale = gimp_scale_entry_new (hbox,
 							   0.0, 
 							   -100.0, 100.0,
 							   1.0,
 							   10.0);
 
-	data->viewer = IMAGE_VIEWER (window->viewer);
+	data->viewer = gth_window_get_image_viewer (window);
+
+	reset_image = glade_xml_get_widget (data->gui, "hl_reset_image");
+	gtk_image_set_from_stock (GTK_IMAGE (reset_image), GTHUMB_STOCK_RESET, GTK_ICON_SIZE_MENU);
 
 	/**/
 
@@ -277,52 +292,44 @@ dlg_hue_saturation (GThumbWindow *window)
 	image_width = gdk_pixbuf_get_width (image);
 	image_height = gdk_pixbuf_get_height (image);
 
-	if (image_width > PREVIEW_SIZE || image_height > PREVIEW_SIZE) {
-		double factor;
+	preview_width  = image_width;
+	preview_height = image_height;
+	scale_keeping_ratio (&preview_width, &preview_height, PREVIEW_SIZE, PREVIEW_SIZE, FALSE);
 
-		factor = MIN ((double) (PREVIEW_SIZE) / image_width, 
-			      (double) (PREVIEW_SIZE) / image_height);
-		preview_width  = MAX ((int) (factor * image_width), 1);
-		preview_height = MAX ((int) (factor * image_height), 1);
-	} else {
-		preview_width  = image_width;
-		preview_height = image_height;
-	}
-	
-	data->orig_pixbuf = gdk_pixbuf_scale_simple (image, 
+	data->orig_pixbuf = gdk_pixbuf_scale_simple (image,
 						     preview_width, 
 						     preview_height,
 						     GDK_INTERP_BILINEAR);
 	data->new_pixbuf = gdk_pixbuf_copy (data->orig_pixbuf);
 
-	gtk_image_set_from_pixbuf (GTK_IMAGE (data->bc_preview_image), 
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->bc_preview_image),
 				   data->new_pixbuf);
-	
+
 	/* Set widgets data. */
 
 	/* Set the signals handlers. */
-	
+
 	g_signal_connect (G_OBJECT (data->dialog),
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (cancel_button), 
+	g_signal_connect (G_OBJECT (cancel_button),
 			  "clicked",
 			  G_CALLBACK (cancel_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (ok_button), 
+	g_signal_connect (G_OBJECT (ok_button),
 			  "clicked",
 			  G_CALLBACK (ok_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (reset_button), 
+	g_signal_connect (G_OBJECT (reset_button),
 			  "clicked",
 			  G_CALLBACK (reset_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (preview_button), 
+	g_signal_connect (G_OBJECT (preview_button),
 			  "clicked",
 			  G_CALLBACK (preview_cb),
 			  data);
@@ -347,8 +354,9 @@ dlg_hue_saturation (GThumbWindow *window)
 	/* Run dialog. */
 
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog),
-				      GTK_WINDOW (window->app));
+				      GTK_WINDOW (window));
+	gtk_window_set_modal (GTK_WINDOW (data->dialog), TRUE);
 	gtk_widget_show (data->dialog);
 
-	apply_changes (data, data->orig_pixbuf, data->new_pixbuf, TRUE);
+	apply_changes (data, data->orig_pixbuf, data->new_pixbuf, TRUE, FALSE);
 }
