@@ -35,6 +35,8 @@
 #include "gth-exif-utils.h"
 #include "gth-exif-data-viewer.h"
 #include "image-viewer.h"
+#include "gconf-utils.h"
+#include "preferences.h"
 
 #include <libexif/exif-data.h>
 #include <libexif/exif-content.h>
@@ -58,17 +60,17 @@ typedef enum {
 } GthMetadataCategory;
 
 
-static char *metadata_category_name[GTH_METADATA_CATEGORIES] = 
+static char *metadata_category_name[GTH_METADATA_CATEGORIES] =
 {
 	N_("Filesystem Data"),
-       	N_("General Information"), 
-	N_("Picture-Taking Conditions"), 
-	N_("Maker Notes"), 
-	N_("GPS Coordinates"), 
-	N_("Image Structure"), 
-	N_("Embedded Thumbnail"), 
-	N_("Versions & Interoperability"), 
-	N_("Other") 
+       	N_("General Information"),
+	N_("Picture-Taking Conditions"),
+	N_("Maker Notes"),
+	N_("GPS Coordinates"),
+	N_("Image Structure"),
+	N_("Embedded Thumbnail"),
+	N_("Versions & Interoperability"),
+	N_("Other")
 };
 
 
@@ -101,7 +103,7 @@ static char *metadata_category_name[GTH_METADATA_CATEGORIES] =
 
 static ExifTag exif_tag_category_map[GTH_METADATA_CATEGORIES][MAX_TAGS_PER_CATEGORY] = {
 
-	/* GTH_METADATA_CATEGORY_FILE */ 
+	/* GTH_METADATA_CATEGORY_FILE */
 	/* The filesystem info is generated and inserted by gthumb in the
 	   correct order, so we don't need to sort it. */
 	{ -1 },
@@ -130,7 +132,7 @@ static ExifTag exif_tag_category_map[GTH_METADATA_CATEGORIES][MAX_TAGS_PER_CATEG
 	  EXIF_TAG_RELATED_SOUND_FILE,
 	  -1 },
 
-        /* GTH_METADATA_CATEGORY_EXIF_CONDITIONS */
+	/* GTH_METADATA_CATEGORY_EXIF_CONDITIONS */
 	/* These tags describe the conditions when the photo was taken,
 	   and are located in the IFD0 or EXIF blocks. */
 	{ EXIF_TAG_ISO_SPEED_RATINGS,
@@ -182,14 +184,14 @@ static ExifTag exif_tag_category_map[GTH_METADATA_CATEGORIES][MAX_TAGS_PER_CATEG
 
 	  EXIF_TAG_DEVICE_SETTING_DESCRIPTION,
 	  EXIF_TAG_OECF,
-	  EXIF_TAG_SPATIAL_FREQUENCY_RESPONSE, 
+	  EXIF_TAG_SPATIAL_FREQUENCY_RESPONSE,
 	  EXIF_TAG_SPECTRAL_SENSITIVITY,
 	  EXIF_TAG_GAIN_CONTROL,
 
 	  -1 },
 
-	/* GTH_METADATA_CATEGORY_MAKERNOTE */ 
-	/* These tags are semi-proprietary and vary from manufacturer to 
+	/* GTH_METADATA_CATEGORY_MAKERNOTE */
+	/* These tags are semi-proprietary and vary from manufacturer to
 	   manufacturer, so we don't bother trying to sort them. They are
 	   listed in the order that they appear in the jpeg file. */
 	{ -1 },
@@ -232,7 +234,7 @@ static ExifTag exif_tag_category_map[GTH_METADATA_CATEGORIES][MAX_TAGS_PER_CATEG
 	/* GTH_METADATA_CATEGORY_EXIF_IMAGE */
 	/* These tags describe the main image data structures, and
 	   come from the IFD0 and EXIF blocks. */
-	{ 
+	{
 
 	  // Image data structure
 
@@ -279,11 +281,11 @@ static ExifTag exif_tag_category_map[GTH_METADATA_CATEGORIES][MAX_TAGS_PER_CATEG
 
 	  -1 },
 
-	/* GTH_METADATA_CATEGORY_EXIF_THUMBNAIL */ 
+	/* GTH_METADATA_CATEGORY_EXIF_THUMBNAIL */
 	/* There are normally only a few of these tags, so we don't bother
 	   sorting them. They are displayed in the order that they appear in
 	   the file. IFD0 (main image) and IFD1 (thumbnail) share many of the
-	   same tags. The IFD0 tags are sorted by the structures above. The 
+	   same tags. The IFD0 tags are sorted by the structures above. The
 	   IFD1 tags are placed into this category. */
 	{ -1 },
 
@@ -294,11 +296,11 @@ static ExifTag exif_tag_category_map[GTH_METADATA_CATEGORIES][MAX_TAGS_PER_CATEG
 	  EXIF_TAG_INTEROPERABILITY_INDEX,
 	  EXIF_TAG_INTEROPERABILITY_VERSION,
 	  EXIF_TAG_RELATED_IMAGE_FILE_FORMAT,
-          EXIF_TAG_RELATED_IMAGE_WIDTH,
-          EXIF_TAG_RELATED_IMAGE_LENGTH,
+	  EXIF_TAG_RELATED_IMAGE_WIDTH,
+	  EXIF_TAG_RELATED_IMAGE_LENGTH,
 	  -1 },
 
-	/* GTH_METADATA_CATEGORY_OTHER */ 
+	/* GTH_METADATA_CATEGORY_OTHER */
 	/* New and unrecognized tags automatically go here. */
 	{ -1 }
 };
@@ -314,7 +316,7 @@ enum {
 
 struct _GthExifDataViewerPrivate
 {
-	char                *path;
+	FileData            *file;
 	gboolean             view_file_info;
 	GtkWidget           *scrolled_win;
 	GtkWidget           *image_exif_view;
@@ -335,8 +337,8 @@ gth_exif_data_viewer_destroy (GtkObject *object)
 	edv = GTH_EXIF_DATA_VIEWER (object);
 
 	if (edv->priv != NULL) {
-		g_free (edv->priv->path);
-		edv->priv->path = NULL;
+		file_data_unref (edv->priv->file);
+		edv->priv->file = NULL;
 		g_free (edv->priv);
 		edv->priv = NULL;
 	}
@@ -383,7 +385,7 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 	gtk_container_add (GTK_CONTAINER (edv), edv->priv->scrolled_win);
 
 	edv->priv->image_exif_view = gtk_tree_view_new ();
-	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (edv->priv->image_exif_view), TRUE);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (edv->priv->image_exif_view), FALSE);
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (edv->priv->image_exif_view), TRUE);
 	edv->priv->image_exif_model = gtk_tree_store_new (3,
 							  G_TYPE_STRING,
@@ -397,7 +399,7 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 	/**/
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Field"),
+	column = gtk_tree_view_column_new_with_attributes ("",
 							   renderer,
 							   "text", NAME_COLUMN,
 							   NULL);
@@ -409,7 +411,7 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 	/**/
 
 	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Value "),
+	column = gtk_tree_view_column_new_with_attributes ("",
 							   renderer,
 							   "text", VALUE_COLUMN,
 							   NULL);
@@ -429,12 +431,12 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 
 
 GType
-gth_exif_data_viewer_get_type ()
+gth_exif_data_viewer_get_type (void)
 {
-        static GType type = 0;
+	static GType type = 0;
 
-        if (! type) {
-                GTypeInfo type_info = {
+	if (! type) {
+		GTypeInfo type_info = {
 			sizeof (GthExifDataViewerClass),
 			NULL,
 			NULL,
@@ -452,7 +454,7 @@ gth_exif_data_viewer_get_type ()
 					       0);
 	}
 
-        return type;
+	return type;
 }
 
 
@@ -533,10 +535,10 @@ add_to_exif_display_list (GthExifDataViewer   *edv,
 		GtkTreePath *path;
 		gtk_tree_store_append (edv->priv->image_exif_model, &root_iter, NULL);
 		gtk_tree_store_set (edv->priv->image_exif_model, &root_iter,
-        		    	    NAME_COLUMN, _(metadata_category_name[category]),
-			    	    VALUE_COLUMN, "",
-			    	    POS_COLUMN, category,
-                            	    -1);
+				    NAME_COLUMN, _(metadata_category_name[category]),
+				    VALUE_COLUMN, "",
+				    POS_COLUMN, category,
+				    -1);
 		path = gtk_tree_model_get_path (model, &root_iter);
 		edv->priv->category_root[category] = gtk_tree_row_reference_new (model, path);
 		gtk_tree_path_free (path);
@@ -551,11 +553,11 @@ add_to_exif_display_list (GthExifDataViewer   *edv,
 	}
 
 	gtk_tree_store_append (edv->priv->image_exif_model, &iter, &root_iter);
-        gtk_tree_store_set (edv->priv->image_exif_model, &iter,
-        		    NAME_COLUMN, utf8_name,
+	gtk_tree_store_set (edv->priv->image_exif_model, &iter,
+			    NAME_COLUMN, utf8_name,
 			    VALUE_COLUMN, utf8_value,
 			    POS_COLUMN, position,
-                            -1);
+			    -1);
 }
 
 
@@ -576,7 +578,7 @@ tag_category (ExifTag  tag,
 		/* Data in IFD1 is for the embedded thumbnail. Keep it separate, do not sort */
 		return GTH_METADATA_CATEGORY_EXIF_THUMBNAIL;
 		break;
-	
+
 	case EXIF_IFD_GPS:
 		/* Go straight to the GPS category if this is in a GPS IFD, to
 		   avoid the tag ID overlap problem. Do sort. */
@@ -615,41 +617,39 @@ static void
 update_exif_data (GthExifDataViewer *edv,
 		  ExifData          *edata)
 {
-	const char   *path;
 	unsigned int  i, j, unique_id_for_unsorted_tags;
 	gboolean      list_is_empty = TRUE;
 
-	path = get_file_path_from_uri (edv->priv->path);
-	if (path == NULL)
+	if (edv->priv->file == NULL)
 		return;
-
-	if (edata == NULL)
-		edata = exif_data_new_from_file (path);
+ 
+	if (edata == NULL) 
+		edata = gth_exif_data_new_from_uri (edv->priv->file->path);
 	else
 		exif_data_ref (edata);
 
 	if (edata == NULL)
-                return;
+		return;
 
 	/* Iterate through every IFD in the Exif data, checking for tags. The GPS tags are
 	   stored in their own private IFD. */
 
 	unique_id_for_unsorted_tags = MAX_TAGS_TOTAL_INCLUDING_MAKERNOTES;
 
-        for (i = 0; i < EXIF_IFD_COUNT; i++) {
-                ExifContent *content = edata->ifd[i];
+	for (i = 0; i < EXIF_IFD_COUNT; i++) {
+		ExifContent *content = edata->ifd[i];
 		const char  *value;
 		char        *utf8_name;
 		char        *utf8_value;
 
-                if (! edata->ifd[i] || ! edata->ifd[i]->count)
-                        continue;
+		if (! edata->ifd[i] || ! edata->ifd[i]->count)
+			continue;
 
-                for (j = 0; j < content->count; j++) {
-                        ExifEntry *e = content->entries[j];
+		for (j = 0; j < content->count; j++) {
+			ExifEntry *e = content->entries[j];
 
-                        if (! content->entries[j])
-                                continue;
+			if (! content->entries[j])
+				continue;
 
 			/* Accept all tags, but handle "maker notes" separately below */
 
@@ -719,33 +719,33 @@ update_exif_data (GthExifDataViewer *edv,
 
 				/* Iterate through each "sub-note" */
 				for (k = 0; k < subnote_count; k++) {
-	                               	value = exif_mnote_data_get_title (mnote, k);
+					value = exif_mnote_data_get_title (mnote, k);
 
-        	                       	if (value == NULL)
-                	                	continue;
+					if (value == NULL)
+						continue;
 
-	      	                       	utf8_name = g_strdup (value);
-                	               	if (tag_is_present_in_category (edv, GTH_METADATA_CATEGORY_MAKERNOTE, utf8_name)) {
-                        	               	g_free (utf8_name);
-                                	       	continue;
-	                               	}
+					utf8_name = g_strdup (value);
+					if (tag_is_present_in_category (edv, GTH_METADATA_CATEGORY_MAKERNOTE, utf8_name)) {
+						g_free (utf8_name);
+						continue;
+					}
 
-        	                       	exif_mnote_data_get_value (mnote, k, mnote_buf, sizeof (mnote_buf));
+					exif_mnote_data_get_value (mnote, k, mnote_buf, sizeof (mnote_buf));
 
 					utf8_value = g_strdup (mnote_buf);
-	                               	if ((utf8_value == NULL)
-        	                           || (*utf8_value == 0)
-                	                   || _g_utf8_all_spaces (utf8_value)) {
-                        	              	g_free (utf8_name);
-                                	      	g_free (utf8_value);
-                                        	continue;
-	                        	}
+					if ((utf8_value == NULL)
+					   || (*utf8_value == 0)
+					   || _g_utf8_all_spaces (utf8_value)) {
+						g_free (utf8_name);
+						g_free (utf8_value);
+						continue;
+					}
 
 					++unique_id_for_unsorted_tags;
 					add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_MAKERNOTE, utf8_name, utf8_value, unique_id_for_unsorted_tags);
 
-	   	                        g_free (utf8_name);
-		                        g_free (utf8_value);
+					g_free (utf8_name);
+					g_free (utf8_value);
 				}
 			}
 
@@ -761,50 +761,54 @@ static void
 update_file_info (GthExifDataViewer *edv)
 {
 	char              *utf8_name;
+	char		  *utf8_fullname;
 	int                width, height;
 	char              *size_txt;
 	time_t             mtime;
 	struct tm         *tm;
 	char               time_txt[50], *utf8_time_txt;
-	double             sec;
-	GnomeVFSFileSize   file_size;
 	char              *file_size_txt;
+	const char	  *mime_type;
 
 	if (edv->priv->viewer == NULL)
 		return;
 
-	utf8_name = g_filename_display_basename (edv->priv->path);
+	utf8_name = basename_for_display (edv->priv->file->path);
+	utf8_fullname = gnome_vfs_unescape_string_for_display (edv->priv->file->path);
 
 	if (!image_viewer_is_void (IMAGE_VIEWER (edv->priv->viewer))) {
 		width = image_viewer_get_image_width (edv->priv->viewer);
 		height = image_viewer_get_image_height (edv->priv->viewer);
-	} else {
+	} 
+	else {
 		width = 0;
 		height = 0;
 	}
 	size_txt = g_strdup_printf (_("%d x %d pixels"), width, height);
 
-	mtime = get_file_mtime (edv->priv->path);
+	mtime = edv->priv->file->mtime;
 	tm = localtime (&mtime);
 	strftime (time_txt, 50, _("%d %B %Y, %H:%M"), tm);
 	utf8_time_txt = g_locale_to_utf8 (time_txt, -1, 0, 0, 0);
-	sec = g_timer_elapsed (image_loader_get_timer (edv->priv->viewer->loader),  NULL);
 
-	file_size = get_file_size (edv->priv->path);
-	file_size_txt = gnome_vfs_format_file_size_for_display (file_size);
+	file_size_txt = gnome_vfs_format_file_size_for_display (edv->priv->file->size);
 
+	mime_type = edv->priv->file->mime_type;
+	
 	/**/
 
-	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Name"), utf8_name, -6);
+	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Name"), utf8_name, -7);
+	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Path"), utf8_fullname, -6);
 	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Dimensions"), size_txt, -5);
 	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Size"), file_size_txt, -4);
 	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Modified"), utf8_time_txt, -3);
-	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Type"), gnome_vfs_mime_get_description (get_mime_type (edv->priv->path)), -2);
+	add_to_exif_display_list (edv, GTH_METADATA_CATEGORY_FILE, _("Type"), mime_type, -2);
 
 	/**/
 
 	g_free (utf8_time_txt);
 	g_free (utf8_name);
+	g_free (utf8_fullname);
 	g_free (size_txt);
 	g_free (file_size_txt);
 }
@@ -819,27 +823,27 @@ gth_exif_data_viewer_view_file_info (GthExifDataViewer *edv,
 
 
 static void
-set_path (GthExifDataViewer *edv,
-	  const char        *path)
+set_file_data (GthExifDataViewer *edv,
+	       FileData          *fd)
 {
 	g_return_if_fail (edv != NULL);
 
-	g_free (edv->priv->path);
-	edv->priv->path = NULL;
-	if (path != NULL)
-		edv->priv->path = g_strdup (path);
+	file_data_unref (edv->priv->file);
+	edv->priv->file = NULL;
+	if (fd != NULL)
+		edv->priv->file = file_data_ref (fd);
 }
 
 
 void
 gth_exif_data_viewer_update (GthExifDataViewer *edv,
 			     ImageViewer       *viewer,
-			     const char        *path,
+			     FileData          *file_data,
 			     gpointer           exif_data)
 {
 	int i;
 
-	set_path (edv, path);
+	set_file_data (edv, file_data);
 
 	if (viewer != NULL)
 		edv->priv->viewer = viewer;
@@ -851,12 +855,11 @@ gth_exif_data_viewer_update (GthExifDataViewer *edv,
 		}
 	gtk_tree_store_clear (edv->priv->image_exif_model);
 
-	if (path == NULL)
+	if (file_data == NULL)
 		return;
 
 	if (edv->priv->view_file_info)
 		update_file_info (edv);
-
 	update_exif_data (edv, exif_data);
 
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (edv->priv->image_exif_view));
