@@ -295,31 +295,28 @@ add_monitor_event (GthMonitor                *monitor,
 		   const char                *path,
 		   GList                    **monitor_events)
 {
-	MonitorEventType type;
-
+	MonitorEventType  type;
+	char             *op;
+	
 	if (!monitor->priv->monitor_enabled)
 		return;
 
-#ifdef DEBUG
-	{
-		char *op;
+	if (event_type == GNOME_VFS_MONITOR_EVENT_CREATED)
+		op = "CREATED";
+	else if (event_type == GNOME_VFS_MONITOR_EVENT_DELETED)
+		op = "DELETED";
+	else
+		op = "CHANGED";
 
-		if (event_type == GNOME_VFS_MONITOR_EVENT_CREATED)
-			op = "CREATED";
-		else if (event_type == GNOME_VFS_MONITOR_EVENT_DELETED)
-			op = "DELETED";
-		else
-			op = "CHANGED";
-
-		debug (DEBUG_INFO, "[%s] %s", op, path);
-	}
-#endif
+	debug (DEBUG_INFO, "[%s] %s", op, path);
 
 	if (event_type == GNOME_VFS_MONITOR_EVENT_CREATED) {
 		if (path_is_file (path))
 			type = MONITOR_EVENT_FILE_CREATED;
-		else
+		else if (path_is_dir (path))
 			type = MONITOR_EVENT_DIR_CREATED;
+		else
+			return;
 
 	} else if (event_type == GNOME_VFS_MONITOR_EVENT_DELETED) {
 		if (file_is_image_video_or_audio (path, TRUE))
