@@ -26,6 +26,8 @@
 #include <gtk/gtkwindow.h>
 #include "gth-pixbuf-op.h"
 #include "image-viewer.h"
+#include "gthumb-preloader.h"
+#include "file-data.h"
 
 
 #define GTH_TYPE_WINDOW              (gth_window_get_type ())
@@ -51,54 +53,57 @@ struct _GthWindowClass
 
 	/*<virtual functions>*/
 
-	void          (*close)                         (GthWindow *window);
-	ImageViewer * (*get_image_viewer)              (GthWindow   *window);
-	const char *  (*get_image_filename)            (GthWindow   *window);
-	void          (*set_image_modified)            (GthWindow   *window,
-							gboolean     value);
-	gboolean      (*get_image_modified)            (GthWindow   *window);
-	void          (*save_pixbuf)                   (GthWindow   *window,
-							GdkPixbuf   *pixbuf,
-							const char  *filename);
-	void          (*exec_pixbuf_op)                (GthWindow   *window,
-							GthPixbufOp *pixop,
-							gboolean     preview);
-	void          (*reload_current_image)          (GthWindow   *window);
-	void          (*update_current_image_metadata) (GthWindow   *window);
-	GList *       (*get_file_list_selection)       (GthWindow   *window);
-	GList *       (*get_file_list_selection_as_fd) (GthWindow   *window);
-	void          (*set_animation)                 (GthWindow   *window,
-							gboolean     value);
-	gboolean      (*get_animation)                 (GthWindow   *window);
-	void          (*step_animation)                (GthWindow   *window);
-	void          (*set_fullscreen)                (GthWindow   *window,
-							gboolean     value);
-	void          (*set_slideshow)                 (GthWindow   *window,
-							gboolean     value);
+	void             (*close)                         (GthWindow   *window);
+	ImageViewer *    (*get_image_viewer)              (GthWindow   *window);
+	GThumbPreloader *(*get_preloader)                 (GthWindow   *window);
+	FileData *       (*get_image_data)                (GthWindow   *window);
+	void             (*set_image_modified)            (GthWindow   *window,
+							   gboolean     value);
+	gboolean         (*get_image_modified)            (GthWindow   *window);
+	void             (*save_pixbuf)                   (GthWindow   *window,
+							   GdkPixbuf   *pixbuf,
+							   FileData    *file);
+	void             (*exec_pixbuf_op)                (GthWindow   *window,
+							   GthPixbufOp *pixop,
+							   gboolean     preview);
+	void             (*reload_current_image)          (GthWindow   *window);
+	void             (*update_current_image_metadata) (GthWindow   *window);
+	GList *          (*get_file_list_selection)       (GthWindow   *window);
+	GList *          (*get_file_list_selection_as_fd) (GthWindow   *window);
+	void             (*set_animation)                 (GthWindow   *window,
+							   gboolean     value);
+	gboolean         (*get_animation)                 (GthWindow   *window);
+	void             (*step_animation)                (GthWindow   *window);
+	void             (*set_fullscreen)                (GthWindow   *window,
+							   gboolean     value);
+	void             (*set_slideshow)                 (GthWindow   *window,
+							   gboolean     value);
 };
 
-GType          gth_window_get_type                       (void);
-void           gth_window_close                          (GthWindow   *window);
-ImageViewer *  gth_window_get_image_viewer               (GthWindow   *window);
-const char *   gth_window_get_image_filename             (GthWindow   *window);
-void           gth_window_set_image_modified             (GthWindow   *window,
-							  gboolean     value);
-gboolean       gth_window_get_image_modified             (GthWindow   *window);
-GdkPixbuf     *gth_window_get_image_pixbuf               (GthWindow   *window);
-void           gth_window_set_image_pixbuf               (GthWindow   *window,
-							  GdkPixbuf   *pixbuf);
-void           gth_window_save_pixbuf                    (GthWindow   *window,
-							  GdkPixbuf   *pixbuf,
-							  const char  *filename);
-void           gth_window_exec_pixbuf_op                 (GthWindow   *window,
-							  GthPixbufOp *pixop,
-							  gboolean     preview);
+GType             gth_window_get_type                     (void);
+void              gth_window_close                        (GthWindow   *window);
+ImageViewer *     gth_window_get_image_viewer             (GthWindow   *window);
+GThumbPreloader * gth_window_get_preloader                (GthWindow   *window);
+FileData *        gth_window_get_image_data               (GthWindow   *window);
+const char *      gth_window_get_image_filename           (GthWindow   *window);
+void              gth_window_set_image_modified           (GthWindow   *window,
+							   gboolean     value);
+gboolean          gth_window_get_image_modified           (GthWindow   *window);
+GdkPixbuf     *   gth_window_get_image_pixbuf             (GthWindow   *window);
+void              gth_window_set_image_pixbuf             (GthWindow   *window,
+							   GdkPixbuf   *pixbuf);
+void              gth_window_save_pixbuf                  (GthWindow   *window,
+							   GdkPixbuf   *pixbuf,
+							   FileData    *file);
+void              gth_window_exec_pixbuf_op               (GthWindow   *window,
+							   GthPixbufOp *pixop,
+							   gboolean     preview);
 
-void           gth_window_undo                           (GthWindow   *window);
-void           gth_window_redo                           (GthWindow   *window);
-void           gth_window_clear_undo_history             (GthWindow   *window);
-gboolean       gth_window_get_can_undo                   (GthWindow   *window);
-gboolean       gth_window_get_can_redo                   (GthWindow   *window);
+void              gth_window_undo                         (GthWindow   *window);
+void              gth_window_redo                         (GthWindow   *window);
+void              gth_window_clear_undo_history           (GthWindow   *window);
+gboolean          gth_window_get_can_undo                 (GthWindow   *window);
+gboolean          gth_window_get_can_redo                 (GthWindow   *window);
 
 typedef enum {
 	GTH_WINDOW_MENUBAR,
@@ -110,7 +115,6 @@ typedef enum {
 void           gth_window_attach                         (GthWindow     *window,
 							  GtkWidget     *child,
 							  GthWindowArea  area);
-
 void           gth_window_set_categories_dlg             (GthWindow   *window,
 							  GtkWidget   *dialog);
 GtkWidget *    gth_window_get_categories_dlg             (GthWindow   *window);
@@ -122,19 +126,15 @@ void           gth_window_reload_current_image           (GthWindow   *window);
 void           gth_window_update_current_image_metadata  (GthWindow   *window);
 GList *        gth_window_get_file_list_selection        (GthWindow   *window);
 GList *        gth_window_get_file_list_selection_as_fd  (GthWindow   *window);
-
 void           gth_window_set_animation                  (GthWindow   *window,
 							  gboolean     value);
 gboolean       gth_window_get_animation                  (GthWindow   *window);
 void           gth_window_step_animation                 (GthWindow   *window);
-
 void           gth_window_edit_comment                   (GthWindow   *window);
 void           gth_window_edit_categories                (GthWindow   *window);
-
 void           gth_window_set_fullscreen                 (GthWindow   *window,
 							  gboolean     value);
 gboolean       gth_window_get_fullscreen                 (GthWindow   *window);
-
 void           gth_window_set_slideshow                  (GthWindow   *window,
 							  gboolean     value);
 gboolean       gth_window_get_slideshow                  (GthWindow   *window);
