@@ -23,7 +23,6 @@
 #include <config.h>
 #include <gnome.h>
 #include <glade/glade.h>
-#include <gdk/gdkx.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomeui/gnome-authentication-manager.h>
@@ -209,7 +208,7 @@ convert_to_new_comment_system (void)
 				 TRUE,
 				 convert_old_comment,
 				 NULL);
-	g_print ("done.");
+	g_print ("done.\n");
 	eel_gconf_set_boolean (PREF_MIGRATE_COMMENT_SYSTEM, FALSE);
 }
 
@@ -951,7 +950,7 @@ get_fs_icon (IconName icon_name,
 
 		w = gdk_pixbuf_get_width (icon_pixbuf[icon_name]);
 		h = gdk_pixbuf_get_height (icon_pixbuf[icon_name]);
-		if (scale_keepping_ratio (&w, &h, icon_size, icon_size))
+		if (scale_keeping_ratio (&w, &h, icon_size, icon_size, FALSE))
 			pixbuf = gdk_pixbuf_scale_simple (icon_pixbuf[icon_name],
 							  w,
 							  h,
